@Echo Off
REM
REM GetCSVFileFtp.cmd, version 1.0.
REM -------------------------------
REM V1.1 - Jesper Wachs, September 2016.
REM
REM CMD fil henter en fil fra IBM i over p pc og konverterer den til CSV format undervejs.
REM Endvidere hentes felt beskrivelserne, som benyttes som overskrifter for kolonerne.
REM
REM CMD file har 7 parametre:
REM 
REM 1. (Valgfrit) Library der indeholder filen.
REM 2. (Valgfrit) Navn p filen.
REM 3. (Valgfrit) Sti og navn p pc filen. Sti med mellemrum sttes " rundt om.
REM 4. (Valgfrit) Navn eller IP adresse p IBM i.
REM 5. (Valgfrit) Brugerprofil p IBM i.
REM 6. (Valgfrit) Kodeord p IBM i.
REM 7. (Valgfrit) Skriv en overskrift linje i CSV filen.
REM
REM V1.1, September 2016: Tilfjet et flag for at tillade at undertrykke overskriften
REM                       (feltnavne) da det viser sig, at Command fortolkeren ikke
REM                       kan fortolke felt beskrivelser der indeholder '*', ')' og '('.
REM 
REM V1.0, August 2016: Initiel version.
Cls
SetLocal EnableDelayedExpansion
REM
REM Vrktjs konfiguration.
REM -----------------------
set scriptfile=%~dp0%~n0.script
set logfile=%~dp0%~n0.log
set logfile2=%~dp0%~n02.log
set filelist=%~dp0%~n03.log
set fldhdr=%~dp0%~n04.log
REM
REM Initiel fejlcheck.
REM ------------------
REM Hvis parm 1 er /? -?, vis hjlpetekst.
If "%1" == "/?" Goto Help
If "%1" == "-?" Goto Help
Goto AllGood

:Help
REM
REM Vis hjlpetekst.
REM
Echo.
Echo Syntaks p kommando:
Echo GetCSVFileFtp library  pf/lf_fil  pc_fil  IBM_i  brugerprofil  kodeord  oversk
Echo.
Echo Beskrivelse:
Echo     Henter en fysisk eller logisk fil fra IBM i til pc og konverterer data
Echo     undervejs til CSV format med semikolon og komma som decimal adskiller.
Echo     Endvidere hentes feltbeskrivelser som benyttes som kolonne overskrifter.
Echo.
Echo Parametre:
Echo     library       Library der indeholder filen.
Echo.
Echo     pf/lf         Navn p filen.
Echo.
Echo     pc filnavn    Sti og navn p pc filen.
Echo.
Echo     IBM_i         Navn eller IP adresse p IBM i.
Echo.
Echo     bruger id     Brugerprofil p iSeries.
Echo.
Echo     kodeord       Kodeord p iSeries.
Echo.
Echo     oversk        Flag der indikerer om der skal skrives en overskrift i CSV
Echo                   filen eller ikke.
Pause
Goto End

:AllGood
REM
REM Hvis parameter Library mangler, sprg efter det.
REM
If "%1" == "" (
  Echo.
  set /p lib= Indtast navn p library hvor pf eller lf filen befinder sig:
  ) Else (
  set lib=%1
  )
REM
REM Hvis parameter Fil mangler, sprg efter det.
REM
If "%2" == "" (
  Echo.
  set /p fil= Indtast navn p fil der indeholder data du vil overfre:
  ) Else (
  set fil=%2
  )
REM
REM Hvis parameter PC fil mangler, sprg efter det.
REM
If "%~3" == "" (
  Echo.
  set /p pcfil= Indtast sti og navn p pc filen der skal overfres til:
  ) Else (
  set pcfil=%~3
  )
REM
REM Hvis parameter IBM i navn mangler, sprg efter det.
REM
If "%4" == "" (
  Echo.
  set /p iseriesname= Indtast DNS-navn eller IP adresse til IBM i:
  ) Else (
  set iseriesname=%4
  )
REM
REM Hvis parameter Bruger id mangler, sprg efter det.
REM
If "%5" == "" (
  Echo.
  set /p user= Indtast bruger id til IBM i:
  ) Else (
  set user=%5
  )
REM
REM Hvis parameter Kodeord mangler, sprg efter det.
REM
If "%6" == "" (
  Echo.
  set /p pwd= Indtast password til IBM i:
  ) Else (
  set pwd=%6
  )
REM
REM Hvis parameter Overskrift mangler, sprg efter det.
REM
If "%7" == "" (
  Echo.
  set /p hdr= Skriv overskrifter [j/n]:
  ) Else (
  set hdr=%6
  )
If /i "%hdr%" NEQ "J" set hdr=N
Cls
REM
REM Forbind og test om bruger id og kodeord er ok, samt at host er en IBM i.
REM
Echo.
Echo Tjekker system, bruger id og kodeord.
Echo.
Echo Vent venligst ...
Echo.
REM
REM Opret FTP script der skal udfres p IBM i.
REM
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote system >> "%scriptfile%"
Echo Quit >>"%scriptfile%"
REM
REM Udfr FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Tjek om brugeren blev afvist.
REM
FIND "530 " < "%logfile2%" > nul
If not errorlevel == 1 (
Echo.
Call :ErrHdr
Call :ErrDetLin "Bruger id og/eller kodeord er ikke korrekt for systemet."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
REM
REM Hvis brugeren ikke blev afvist, da m brugeren vre logget p ;-)
REM
FIND "230 " < "%logfile2%" > nul
If errorlevel == 1 (
Echo.
Call :ErrHdr
Call :ErrDetLin "Bruger id kunne ikke logge p systemet."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
REM
REM Tjek at vi er forbundet til en IBM i.
REM
FIND "OS/400" < "%logfile2%" > nul
If errorlevel == 1 (
Call :ErrHdr
Call :ErrDetLin "Systemet er ikke en IBM i."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
Echo.
Echo Verifikation gik godt.
Echo.
REM
REM Tjek filen findes p IBM i.
REM
Echo.
Echo Tjekker om filen %lib%/%fil% findes.
Echo.
Echo Vent venligst ...
Echo.
REM
REM Opret FTP script der skal udfres p IBM i.
REM
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote rcmd chkobj %lib%/%fil% *file aut(*use) >> "%scriptfile%"
Echo Quit >> "%scriptfile%"
REM
REM Udfr FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Tjek om der skete en fejl p CHKOBJ.
REM
FIND "550-" < "%logfile2%" > nul
If errorlevel == 1 Goto C250
Call :ErrHdr
Call :ErrDetLin "Kontrol af, om filen %lib%/%fil% findes, gik ikke godt."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit

:C250
REM
REM Systemet har returneret 250, hvilket er ganske godt.
REM
FIND "250" < "%logfile2%" > nul
if errorlevel == 1 (
Call :ErrHdr
Call :ErrDetLin "Kontrol af, om filen %lib%/%fil% findes, gik ikke godt."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)

:XFer
REM
REM Konverter filen til CSV fil.
REM
If /i "%hdr%" == "N" Goto XFerFile
Echo.
Echo Forbereder filen %lib%/%fil%.
Echo.
Echo Vent venligst ...
Echo.
REM
REM Opret FTP script der skal udfres p IBM i.
REM
Echo open %iseriesname% >"%scriptfile%"
Echo %user% >>"%scriptfile%"
Echo %pwd% >>"%scriptfile%"
Echo QUOTE TIME 240 240 >>"%scriptfile%"
Echo quote rcmd DSPFFD %lib%/%fil% OUTPUT(*OUTFILE) OUTFILE(QTEMP/XYZTEMP) >>"%scriptfile%"
Echo quote rcmd CPYTOIMPF FROMFILE(QTEMP/XYZTEMP) TOSTMF('/tmp/%username%') RCDDLM(*CRLF) DTAFMT(*FIXED) >>"%scriptfile%"
Echo quote rcmd DLTF QTEMP/XYZTEMP >>"%scriptfile%"
Echo asc >>"%scriptfile%"
Echo get /tmp/%username% "%filelist%" >>"%scriptfile%"
Echo quote rcmd DEL '/tmp/%username%' >>"%scriptfile%"
Echo Quit >> "%scriptfile%"
REM
REM Udfr FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Tjek om det gik godt.
REM
FIND "550" < "%logfile2%" > nul
If errorlevel == 1 Goto Cont2
Echo.
Call :ErrHdr
Call :ErrDetLin "Kunne ikke konvertere filen %1/%2."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit

:Cont2
if not exist "%filelist%" (
Call :ErrHdr
Call :ErrDetLin "%filelist% blev ikke oprettet. Kontroller logfiler."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)

:XferFile
Echo.
Echo Henter filen over p pc.
Echo.
Echo Vent venligst ...
REM
REM Opret FTP script der skal udfres p IBM i.
REM
Echo open %iseriesname% >"%scriptfile%"
Echo %user% >>"%scriptfile%"
Echo %pwd% >>"%scriptfile%"
Echo QUOTE TIME 3600 3600 >>"%scriptfile%"
Echo quote rcmd CPYTOIMPF FROMFILE(%lib%/%fil%) TOSTMF('/tmp/%username%') RCDDLM(*CRLF) DTAFMT(*DLM) FLDDLM(';') DECPNT(*COMMA) >>"%scriptfile%"
Echo asc >>"%scriptfile%"
Echo get /tmp/%username% "%pcfil%" >>"%scriptfile%"
Echo quote rcmd DEL '/tmp/%username%' >>"%scriptfile%"
Echo Quit >>"%scriptfile%"
REM
REM Udfr FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Tjek om det gik godt.
REM
FIND "550" < "%logfile2%" > nul
If errorlevel == 1 Goto Cont3
Echo.
Call :ErrHdr
Call :ErrDetLin "Kunne ikke overfre filen %lib%/%fil%."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit

:Cont3
If /i "%hdr%" == "N" Goto XFerDone
REM
REM Konverter felt filen og lav en overskrift.
REM
if exist "%fldhdr%" del "%fldhdr%"
for /F "usebackq tokens=*" %%A in ("%filelist%") do (
set linjen=%%A
REM Hvis du vil have feltnavnet, erstat med 136,10.
set ftext=!linjen:~184,50!
REM
REM Blot en bemrkning: Noget gr galt nr der enten er stjerne eller parantes
REM i ftext, nem det lader vi ligge for nu.
REM
REM Herunder et trick for at udg CRLF i filen.
REM
 <NUL set /p= ""!ftext!";" >> "%fldhdr%"
)
Echo.>> "%fldhdr%"
REM
REM Kopier filerne sammen.
REM
copy "%fldhdr%"+"%pcfil%" "%fldhdr%"
del "%pcfil%"
REM
REM Udled filnavn af den fulde sti.
REM
for %%i in ("%pcfil%") do set pcfil=%%~ni%%~xi
rename "%fldhdr%" "%pcfil%"

:XFerDone
Echo.
Echo Overfrsel er overstet.
Echo.
REM pause
if exist "%logfile%"  del "%logfile%"
if exist "%logfile2%" del "%logfile2%"
if exist "%filelist%" del "%filelist%"

Goto End

:Exit
REM
REM Afslut ved fejl.
REM
Echo.
Echo Kommandoen er afbrudt.
Echo.
Echo En logfil blev oprettet og du finder den her:
Echo.
Echo - "%logfile%"
Echo.
Pause

:End
if exist "%scriptfile%" del "%scriptfile%"
if exist "%logfile2%"   del "%logfile2%"
set lib=
set fil=
set pcfil=
set filtype=
set iseriesname=
set user=
set pwd=
set scriptfile=
set logfile=
set logfile2=
set filelist=
set fldhdr=
set linjen=
set ftext=
set hdr=
Goto :eof

REM
REM Subrutiner
REM ----------
REM
REM Skriv Fejl Header p skrm og i fil.
REM ------------------------------------
:ErrHdr
REM
REM I logfilen skrives konfigurationen samt de vrige informationer.
REM
Echo %~n0 fejl log. > "%logfile%"
Echo --------------------- >> "%logfile%"
Echo Dato/tid: %date% %time% >> "%logfile%"
Echo ======================================================================== >> "%logfile%"
Echo IBM i navn .....: %iseriesname% >> "%logfile%"
Echo Bruger id ......: %user% >> "%logfile%"

Call :ErrDetLin "========================================================================"
Call :ErrDetLin "Fejl opstet."
Call :ErrDetLin "-------------"
Exit /b

REM
REM Skriv fejl detail linje p skrm og i fil.
REM ------------------------------------------
:ErrDetLin Str
if "%~1" == "" Echo.
if not "%~1" == "" Echo %~1

if "%~1" == "" Echo. >> "%logfile%"
if not "%~1" == "" Echo %~1 >> "%logfile%"
Exit /b

REM
REM Skriv fejl footer p skrm og i fil.
REM ------------------------------------
:ErrFtr
Call :ErrDetLin "========================================================================"
Exit /b

REM
REM Skriv FTP fejl log p skrm og i fil.
REM -------------------------------------
:ErrFTPLog
Call :ErrDetLin ""
Call :ErrDetLin "Kontroller loggen herunder:"
Call :ErrDetLin "--------------------------[ Start p log ]------------------------------"
Type "%logfile2%"
Type "%logfile2%" >> "%logfile%"
Call :ErrDetLin "---------------------------[ Slut p log ]-------------------------------"
Exit /b
